#' Find marker genes using limma
#'
#' @param sce A SingleCellExperiment object
#' @param pars parameters passed to the function. For limma this has
#'   the following components:
#'   * `use_voom`: Should the voom method by used? Otherwise is limma is fitted
#'     to the logcounts.
#'   * `test_strategy`: `"pairwise"`
#'   * `treat_lfc`: The lfc against which the treat test should be run. If 0
#'     treat is not run.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_scran <- function(sce, pars) {

  times <- system.time(
    result <- run_limma_impl(sce, pars)
  )

  # Get the elapsed time.
  time <- times[[3]]

  list(time = time, result = result, pars = pars)
}

# Need to mock this up...
run_limma_impl <- function(sce, pars) {

  # This code is based on the code in:
  # http://bioconductor.org/books/release/OSCA/marker-detection.html

  dge <- limma::convertTo(sce)
  uniuqe_clusters <- unique(dge$samples$label)
  all_results <- list()
  all_pairs <- list()
  counter <- 1

  for (x in uniuqe_clusters ) {
    for (y in uniuqe_clusters ) {

      # Avoid redundant comparisons.
      if (x == y) {
        break
      }

      # Factor ordering ensures that 'x' is the not the intercept,
      # so resulting fold changes can be interpreted as x/y.
      subdge <- dge[, dge$samples$label %in% c(x, y)]
      subdge$samples$label <- factor(subdge$samples$label, c(y, x))
      design <- model.matrix(~ label, subdge$samples)

      # No need to normalize as we are using the size factors
      # transferred in the SCE and converted to norm.factors.
      # FIXME: Should we relax this?
      # We also relax the filtering for the lower UMI counts.
      subdge <- subdge[calculateAverage(subdge$counts) > 0.1, ]

      # Standard voom-limma pipeline starts here.
      if (pars$use_voom) {
        v <- limma::voom(subdge, design)
        fit <- limma::lmFit(v, design)
      }
      if (pars$treat_lfc != 0) {
        fit <- limma::treat(fit, lfc = pars$treat_lfc)
        res <- limma::topTreat(fit, n = Inf, sort.by = "none")
      } else {
        fit <- limma::eBayes(fit)
        res <- limma::topTable(fit, number = Inf)
      }

      # Filling out the genes that got filtered out with NA's.
      res <- res[rownames(dge),]
      rownames(res) <- rownames(dge)

      all.results[[counter]] <- res
      all.pairs[[counter]] <- c(x, y)
      counter <- counter + 1

      # Also filling the reverse comparison.
      res$logFC <- -res$logFC
      all.results[[counter]] <- res
      all.pairs[[counter]] <- c(y, x)
      counter <- counter + 1
    }
  }
}

